"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const link_1 = require("./link");
const configstore_1 = __importDefault(require("configstore"));
// eslint-disable-next-line jest/no-mocks-import
const configstore_2 = require("../__mocks__/configstore");
jest.mock('configstore');
const mockConfigstore = configstore_1.default;
const configFileLocation = configstore_2.getConfigstoreLocation();
const pathPython = require.resolve('@cspell/dict-python/cspell-ext.json');
const pathCpp = require.resolve('@cspell/dict-cpp/cspell-ext.json');
const pathHtml = require.resolve('@cspell/dict-html/cspell-ext.json');
const pathCss = require.resolve('@cspell/dict-css/cspell-ext.json');
const python = require.resolve('@cspell/dict-python/cspell-ext.json');
describe('Validate Link.ts', () => {
    beforeEach(() => {
        mockConfigstore.mockClear();
        configstore_2.clearMocks();
        configstore_2.clearData();
    });
    test('listGlobalImports configstore empty', () => {
        const r = link_1.listGlobalImports();
        expect(r).toEqual({
            list: [],
            globalSettings: {
                source: { filename: configFileLocation, name: 'CSpell Configstore' },
            },
        });
        expect(configstore_2.mockSetData).not.toHaveBeenCalled();
    });
    test('listGlobalImports configstore empty import', () => {
        configstore_2.setData({
            import: [],
        });
        const r = link_1.listGlobalImports();
        expect(r).toEqual({
            list: [],
            globalSettings: {
                source: { filename: configFileLocation, name: 'CSpell Configstore' },
                import: [],
            },
        });
    });
    test('listGlobalImports with imports', () => {
        configstore_2.setData({
            import: [python],
        });
        const r = link_1.listGlobalImports();
        expect(r).toEqual(expect.objectContaining({
            list: [
                expect.objectContaining({
                    filename: python,
                    error: undefined,
                }),
            ],
            globalSettings: expect.objectContaining({
                source: { filename: configFileLocation, name: 'CSpell Configstore' },
            }),
        }));
    });
    test('listGlobalImports with import errors', () => {
        const filename = '__not_found_file_.ext';
        configstore_2.setData({
            import: [filename],
        });
        const r = link_1.listGlobalImports();
        expect(r).toEqual(expect.objectContaining({
            list: [
                expect.objectContaining({
                    filename: filename,
                    error: expect.stringContaining('Failed to read config'),
                }),
            ],
            globalSettings: expect.objectContaining({
                source: { filename: configFileLocation, name: 'CSpell Configstore' },
            }),
        }));
        expect(configstore_2.mockSetData).not.toHaveBeenCalled();
    });
    test('addPathsToGlobalImports', () => {
        configstore_2.setData({
            import: [pathPython, pathCss],
        });
        const r = link_1.addPathsToGlobalImports([pathCpp, pathPython, pathHtml]);
        expect(r.resolvedSettings).toHaveLength(3);
        expect(r).toEqual({
            error: undefined,
            success: true,
            resolvedSettings: [
                expect.objectContaining({ filename: pathCpp, error: undefined }),
                expect.objectContaining({ filename: pathPython, error: undefined }),
                expect.objectContaining({ filename: pathHtml, error: undefined }),
            ],
        });
        expect(configstore_2.mockSetData).toHaveBeenCalledWith({
            import: [pathPython, pathCss, pathCpp, pathHtml],
        });
    });
    test('addPathsToGlobalImports with errors', () => {
        const pathNotFound = '__not_found_file_.ext';
        configstore_2.setData({
            import: [pathPython],
        });
        const r = link_1.addPathsToGlobalImports([pathCpp, pathPython, pathNotFound]);
        expect(r.resolvedSettings).toHaveLength(3);
        expect(r).toEqual({
            error: 'Unable to resolve files.',
            success: false,
            resolvedSettings: [
                expect.objectContaining({ filename: pathCpp, error: undefined }),
                expect.objectContaining({ filename: pathPython, error: undefined }),
                expect.objectContaining({
                    filename: pathNotFound,
                    error: expect.stringContaining('Failed to read config'),
                }),
            ],
        });
        expect(configstore_2.mockSetData).not.toHaveBeenCalled();
    });
    test('removePathsFromGlobalImports', () => {
        configstore_2.setData({
            import: [pathCpp, pathPython, pathCss, pathHtml],
        });
        const r = link_1.removePathsFromGlobalImports([pathCpp, '@cspell/dict-css']);
        expect(r).toEqual({
            success: true,
            error: undefined,
            removed: [pathCpp, pathCss],
        });
        expect(configstore_2.mockSetData).toHaveBeenCalledWith({
            import: [pathPython, pathHtml],
        });
    });
    test('removePathsFromGlobalImports with unknown', () => {
        configstore_2.setData({
            import: [pathCpp, pathPython, pathCss, pathHtml],
        });
        const r = link_1.removePathsFromGlobalImports([
            pathCpp,
            '@cspell/dict-unknown',
            'cspell-ext.json',
            '@cspell/dict-html',
            'python/cspell-ext.json',
        ]);
        expect(r).toEqual({
            success: true,
            error: undefined,
            removed: [pathCpp, pathHtml],
        });
        expect(configstore_2.mockSetData).toHaveBeenCalledWith({
            import: [pathPython, pathCss],
        });
    });
    test('removePathsFromGlobalImports with nothing to remove', () => {
        configstore_2.setData({
            import: [pathCpp, pathPython, pathCss, pathHtml],
        });
        const r = link_1.removePathsFromGlobalImports(['@cspell/dict-unknown', 'cspell-ext.json', 'python/cspell-ext.json']);
        expect(r).toEqual({
            success: true,
            error: undefined,
            removed: [],
        });
        expect(configstore_2.mockSetData).not.toHaveBeenCalled();
    });
});
//# sourceMappingURL=link.test.js.map