"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const wordSplitter_1 = require("./wordSplitter");
const generateWordBreaks = wordSplitter_1.__testing__.generateWordBreaks;
const words = sampleWordSet();
const regHasLetters = /\p{L}/u;
describe('Validate wordSplitter', () => {
    // cspell:ignore MOVSX
    test.each `
        text                | expected
        ${'hello'}          | ${['hello']}
        ${'well-educated'}  | ${['well', 'educated']}
        ${'ERRORCode'}      | ${['ERROR', 'Code']}
        ${'MOVSX_r_rm16'}   | ${['MOVSX', 'r', 'rm']}
        ${'32bit-checksum'} | ${['bit', 'checksum']}
        ${'camelCase'}      | ${['camel', 'Case']}
    `('Apply word breaks to $text', ({ text, expected }) => {
        const line = {
            text,
            offset: 42,
        };
        const lineSeg = {
            line: line,
            relStart: 0,
            relEnd: text.length,
        };
        const posBreaks = generateWordBreaks(lineSeg, {});
        const breaks = extractBreaks(posBreaks);
        const r = applyWordBreaks(line, breaks);
        expect(r.map((t) => t.text)).toEqual(expected);
    });
    test.each `
        text                | expected
        ${'hello'}          | ${['hello']}
        ${'well-educated'}  | ${['well|educated', 'well|-educated', 'well-|educated', 'well-educated']}
        ${'MOVSX_r_rm16'}   | ${['MOVSX|r|rm']}
        ${'32bit-checksum'} | ${['bit|checksum']}
        ${'ERRORCode'}      | ${['ERROR|Code']}
        ${'camelCase'}      | ${['camel|Case', 'camelCase']}
    `('Apply word breaks to $text', ({ text, expected }) => {
        const line = {
            text,
            offset: 42,
        };
        const lineSeg = {
            line: line,
            relStart: 0,
            relEnd: text.length,
        };
        const posBreaks = generateWordBreaks(lineSeg, {});
        const r = genAllPossibleResults(text, posBreaks);
        expect(r[0]).toEqual(expected[0]); // Expect the first candidate to be the one with the shortest words.
        expect(r).toEqual(expect.arrayContaining(expected));
        expect(r).toEqual(expect.arrayContaining([text])); // this assumes the original text is a candidate
        expect(r).toMatchSnapshot(); // Use snapshots to ensure all possible options are generated.
    });
    function tov(p, isValid = true) {
        if (typeof p === 'string') {
            p = { text: p };
        }
        const { isFound = isValid } = p;
        return { ...p, isFound };
    }
    function splitTov(t) {
        if (!t)
            return [];
        const parts = t.split('|');
        return parts.map((p) => tov(p, has({ text: p, offset: 0 })));
    }
    // cspell:ignore CVTPD CVTSI CVTTSD words'separated'by errorcode
    test.each `
        text                                | expectedWords
        ${'hello'}                          | ${[tov({ text: 'hello', offset: 155 })]}
        ${'well-educated'}                  | ${[tov('well'), tov('educated')]}
        ${'MOVSX_r_rm16'}                   | ${splitTov('MOVSX_r_rm16')}
        ${'32bit-checksum'}                 | ${splitTov('bit|checksum')}
        ${'ERRORCode'}                      | ${splitTov('ERROR|Code')}
        ${'camelCase'}                      | ${splitTov('camel|Case')}
        ${'CVTPD2PS_x_xm'}                  | ${splitTov('CVTPD2PS|x|xm')}
        ${'CVTSI2SD_x_rm'}                  | ${splitTov('CVTSI|SD|x|rm')}
        ${'CVTTSD2SI_r_xm'}                 | ${splitTov('CVTTSD|SI|r|xm')}
        ${'error_code42_one_two'}           | ${splitTov('error|code42|one|two')}
        ${'_errorcode42_one_two'}           | ${splitTov('_errorcode42|one|two')}
        ${"words'separated'by_singleQuote"} | ${splitTov(`words'separated'by|singleQuote`)}
        ${"Tom's_hardware"}                 | ${splitTov("Tom's|hardware")}
    `('split $text', ({ text, expectedWords }) => {
        const prefix = 'this is some';
        const line = {
            text: `${prefix} ${text} to split.`,
            offset: 142,
        };
        const offset = line.offset + prefix.length;
        const r = wordSplitter_1.split(line, offset, has);
        expect(r.offset).toBe(offset);
        expect(r.endOffset).toBe(r.text.offset + r.text.text.length);
        expect(r.words).toEqual(expect.arrayContaining(expectedWords.map(expect.objectContaining)));
        expect(r.words).toHaveLength(expectedWords.length);
    });
    test.each `
        text                                | expectedWords                                 | calls
        ${'hello'}                          | ${[tov({ text: 'hello', offset: 142 })]}      | ${1}
        ${''}                               | ${[]}                                         | ${0}
        ${'#@()&*'}                         | ${[]}                                         | ${0}
        ${'well-educated'}                  | ${[tov('well'), tov('educated')]}             | ${2}
        ${'MOVSX_r_rm16'}                   | ${splitTov('MOVSX_r_rm16')}                   | ${8}
        ${'32bit-checksum'}                 | ${splitTov('bit|checksum')}                   | ${2}
        ${'ERRORCodesTwo'}                  | ${splitTov('ERROR|Codes|Two')}                | ${4}
        ${'camelCase'}                      | ${splitTov('camel|Case')}                     | ${2}
        ${'CVTPD2PS_x_xm'}                  | ${splitTov('CVTPD2PS|x|xm')}                  | ${6}
        ${'CVTSI2SD_x_rm'}                  | ${splitTov('CVTSI|SD|x|rm')}                  | ${10}
        ${'errCVTTSD2SI_r_xm'}              | ${splitTov('err|CVTTSD|SI|r|xm')}             | ${12}
        ${"words'separated'by_singleQuote"} | ${splitTov('words|separated|by|singleQuote')} | ${6}
        ${"Tom's_hardware"}                 | ${splitTov("Tom's|hardware")}                 | ${4}
    `('split edge cases `$text`', ({ text, expectedWords, calls }) => {
        const line = {
            text,
            offset: 142,
        };
        const offset = line.offset;
        const h = jest.fn();
        const hasCalls = [];
        h.mockImplementation((t) => {
            hasCalls.push(t.text);
            return has(t);
        });
        const r = wordSplitter_1.split(line, offset, h, { optionalWordBreakCharacters: `'` });
        // console.log(hasCalls);
        expect(r.offset).toBe(offset);
        expect(r.endOffset).toBe(r.text.offset + r.text.text.length);
        expect(r.words).toEqual(expect.arrayContaining(expectedWords.map(expect.objectContaining)));
        expect(r.words).toHaveLength(expectedWords.length);
        expect(h).toHaveBeenCalledTimes(calls);
    });
    // cspell:ignore nstatic techo n'cpp n'log refactor'd
    test.each `
        text            | expectedWords | calls
        ${'static'}     | ${'static'}   | ${1}
        ${'nstatic'}    | ${'static'}   | ${1}
        ${'techo'}      | ${'echo'}     | ${1}
        ${`n'cpp`}      | ${'cpp'}      | ${1}
        ${`n'log`}      | ${'log'}      | ${4}
        ${'64-bit'}     | ${'bit'}      | ${1}
        ${'128-bit'}    | ${'bit'}      | ${1}
        ${'256-sha'}    | ${'256-sha'}  | ${6}
        ${`REFACTOR'd`} | ${'REFACTOR'} | ${2}
    `('split `$text` in doc', ({ text, expectedWords, calls }) => {
        const expectedWordSegments = splitTov(expectedWords);
        const doc = sampleText();
        const line = findLine(doc, text);
        const offset = line.offset + line.text.indexOf(text);
        expect(offset).toBeGreaterThan(0);
        const h = jest.fn();
        const hasCalls = [];
        h.mockImplementation((t) => {
            hasCalls.push(t.text);
            return has(t);
        });
        const r = wordSplitter_1.split(line, offset, h);
        // console.log(hasCalls);
        expect(r.endOffset).toBe(r.text.offset + r.text.text.length);
        expect(r.words).toEqual(expect.arrayContaining(expectedWordSegments.map(expect.objectContaining)));
        expect(r.words).toHaveLength(expectedWordSegments.length);
        expect(h).toHaveBeenCalledTimes(calls);
    });
});
function has({ text }) {
    return text.length < 3 || !regHasLetters.test(text) || words.has(text) || words.has(text.toLowerCase());
}
function applyWordBreaks(text, breaks) {
    const a = text.offset;
    const t = text.text;
    const words = [];
    let i = 0;
    for (let p = 0; p < breaks.length; p += 2) {
        const start = breaks[p];
        const end = breaks[p + 1];
        if (i !== start) {
            words.push({
                text: t.slice(i, start),
                offset: a + i,
            });
        }
        i = end;
    }
    if (i < t.length) {
        words.push({
            text: t.slice(i),
            offset: a + i,
        });
    }
    return words;
}
function extractBreaks(pwb) {
    const r = [];
    for (const b of pwb) {
        const br = b.breaks[0];
        if (br) {
            r.splice(r.length, 0, ...br);
        }
    }
    return r;
}
function genAllPossibleResults(text, breaks) {
    function* genResults(i, bi) {
        const br = breaks[bi];
        if (!br) {
            yield text.slice(i);
            return;
        }
        const emitted = new Set();
        for (const b of br.breaks) {
            const parts = [];
            let p = i;
            for (let x = 0; x < b.length; x += 2) {
                const s = b[x];
                const e = b[x + 1];
                parts.push(text.slice(p, s));
                p = e;
            }
            const prefix = parts.join('|');
            const prefix2 = prefix + (prefix ? '|' : '');
            for (const r of genResults(p, bi + 1)) {
                const t = r ? prefix2 + r : prefix;
                if (!emitted.has(t)) {
                    emitted.add(t);
                    yield t;
                }
            }
        }
    }
    return [...genResults(0, 0)];
}
function findLine(doc, text) {
    const index = doc.indexOf(text);
    let lastLine = undefined;
    for (const line of doc.matchAll(/.*/g)) {
        if ((line.index || 0) < index) {
            lastLine = line;
        }
    }
    if (lastLine && lastLine.index) {
        return {
            text: lastLine[0],
            offset: lastLine.index,
        };
    }
    return {
        text: '',
        offset: -1,
    };
}
function sampleWordSet() {
    const words = `
    _errorcode42
    2SD
    64-bit
    bit checksum
    camel case
    can't
    code42
    const
    cpp
    CVTPD2PS
    CVTTSD
    echo
    error codes
    hello
    MOVSX_r_rm16
    one two
    static
    these are some sample words
    Tom's hardware
    well educated
    words separated by singleQuote
    256-sha
    `
        .split(/\s+/g)
        .map((a) => a.trim())
        .filter((a) => !!a);
    return new Set(words);
}
function sampleText() {
    return `
    static const char new_stub1_1[] = "\\techo"
    \\n'log' => 'text/plain'
	static const char new_stub2[] = "';\\nconst LEN = ";
    static const char new_stub3_0[] = ";\\n\\nstatic function go($return = false)\\n'cpp'"

    /* The escape was a back (or forward) reference. We keep the offset in
    order to give a more useful diagnostic for a bad forward reference. For
    references to groups numbered less than 10 we can't use more than two items
    in parsed_pattern because they may be just two characters in the input (and
    in a 64-bit world an offset may need two elements). So for them, the offset
    ERROR's
    REFACTOR'd
    of the first occurrent is held in a special vector. */

    256-sha

    128-bit values


`;
}
//# sourceMappingURL=wordSplitter.test.js.map